#include <wijit.h>

extern GladeXML *WIJIT__XML;
GladeXML *conference__xml;
GladeXML *join__dialog;

struct conf_list_struct;

typedef struct conf_struct
{
    xode_pool p;
    char *server;
    char *channel;
    char *nick;
    char *jid;
    wresult wid;
    xode conf_result;
    xode browse_result;
    GladeXML *xml;
    struct conf_list_struct *list;
} _conf_data, *conf_data;

typedef struct conf_list_struct
{
    conf_data cdata;
    struct conf_list_struct *next;
    struct conf_list_struct *prev;
} _conf_data_list, *conf_data_list;
conf_data_list cdata__list = NULL;

void _display_text(const char *user, const char *text, conf_data cdata)
{
    GtkWidget *w_text = glade_xml_get_widget(cdata->xml, "conference_text");
    gtk_text_set_point(GTK_TEXT(w_text), gtk_text_get_length(GTK_TEXT(w_text)));

    if(user != NULL)
    {
        gtk_text_insert(GTK_TEXT(w_text), NULL, NULL, NULL, "<", 1);
        gtk_text_insert(GTK_TEXT(w_text), NULL, NULL, NULL, user, strlen(user));
        gtk_text_insert(GTK_TEXT(w_text), NULL, NULL, NULL, "> ", 2);
    }
    else
    {
        gtk_text_insert(GTK_TEXT(w_text), NULL, NULL, NULL, " \%\% ", 4);
    }
    gtk_text_insert(GTK_TEXT(w_text), NULL, NULL, NULL, text, strlen(text));
    gtk_text_insert(GTK_TEXT(w_text), NULL, NULL, NULL, "\n", 2);
}

void _send_message(GtkWidget *w, gpointer data)
{
    conf_data cdata = (conf_data)data;
    char *text = gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(cdata->xml, "conference_entry")));
    wijit_write("<message type='groupchat' to='%s@%s'><body>%s</body></message>", cdata->channel, cdata->server, text);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(cdata->xml, "conference_entry")), "");
}

void _part_chat_room(GtkWidget *w, gpointer data)
{
    conf_data cdata = (conf_data)data;
    conf_data_list list = cdata->list;
    GtkWidget *tab;
    gint page_num;

    /* unlink */
    if(list->next != NULL) list->next->prev = list->prev;
    if(list->prev != NULL) list->prev->next = list->next;

    tab = glade_xml_get_widget(cdata->xml, "conference_interface");
    page_num = gtk_notebook_page_num(GTK_NOTEBOOK(glade_xml_get_widget(WIJIT__XML, "wijit_notebook")), tab);
    if(page_num == -1)
        return;
    gtk_notebook_remove_page(GTK_NOTEBOOK(glade_xml_get_widget(WIJIT__XML, "wijit_notebook")), page_num);
    wijit_unregister(cdata->wid);
    wijit_write("<presence to='%s@%s' type='unavailable'/>", cdata->channel, cdata->server);
}

wresult __join_dialog_join_clicked(const char *id, wevent evt, xode x, void *arg)
{
    /* phase 2 */
    /* this will be called only for conferences that match us */
    conf_data cdata = (conf_data)arg;
    GtkWidget *notebook, *tab, *object;

    if(evt & WE_TAG_ID)
    { /* this should be our iq:result */
        GladeXML *xml;
        xode_pool p;
        if(*xode_get_attrib(x, "type") == 'e')
        {
            xode_pool_free(cdata->p);
            wijit_status("conference", "could not join conference: %s", xode_get_tagdata(x, "error"));
            return WR_OK;
        }

        p = xode_pool_new();
        xml = glade_xml_new(xode_spool_str(p, PLUGIN_DIR, "/conference.glade", p), "conference_interface");
        xode_pool_free(p);
        cdata->conf_result = x;

        /* display our chat dialog */
        /* attach the main vbox to the notebook */
        notebook = glade_xml_get_widget(WIJIT__XML, "wijit_notebook");
        object = gtk_label_new(cdata->channel);
        tab = glade_xml_get_widget(xml, "conference_interface");
        gtk_widget_show(object);
        gtk_widget_show(tab);
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tab, object);

        object = glade_xml_get_widget(xml, "conference_part");
        glade_xml_signal_connect_data(xml, "on_conference_part_clicked", GTK_SIGNAL_FUNC(_part_chat_room), (gpointer)cdata);
        glade_xml_signal_connect_data(xml, "on_conference_entry_activate", GTK_SIGNAL_FUNC(_send_message), (gpointer)cdata);
        cdata->xml = xml;
        xode_free(x);
        return WR_OK;
    }

    /* could get presence here */
    if(*xode_get_name(x) == 'p')
    {
        xode_free(x);
        return WR_OK;
    }

    /* else, these are roster updates, or messages */
    if(*xode_get_name(x) == 'm')
    { /* message to the room */
        /* first find the user this is for */
        xode browse = xode_get_tag(cdata->browse_result, "conference") ? xode_get_tag(cdata->browse_result, "conference"): cdata->browse_result;
        xode cur = xode_get_tag(browse, xode_spool_str(xode_get_pool(x), "user?jid=", xode_get_attrib(x, "from"), xode_get_pool(x)));
        _display_text(xode_get_attrib(cur, "name"), xode_get_tagdata(x, "body"), cdata);
        xode_free(x);
        return WR_OK;
    }

    /* dunno why we'd get these here.. maybe nick change? */
    if(strcmp(xode_get_attrib(xode_get_firstchild(x), "xmlns"), "jabber:iq:conference") == 0)
    {
        xode_free(x);
        return WR_OK;
    }


    /* okay.. this must be roster updates.. */
    if(strcmp(xode_get_attrib(x, "type"), "set") == 0)
    { /* this is a roster update */
        /* first, set the cdata->browse_result */
        xode cur;

        cur = xode_get_tag(x, "conference") ? xode_get_firstchild(xode_get_tag(x, "conference")) : xode_get_firstchild(x);

        /* put all the people in the list */
        for(; cur != NULL; cur = xode_get_nextsibling(cur))
        {
            xode old = xode_get_tag(cdata->browse_result, xode_spool_str(xode_get_pool(x), "user?jid=", xode_get_attrib(cur, "jid"), xode_get_pool(x)));

            /* create the roster container */
            if(cdata->browse_result == NULL)
                cdata->browse_result = xode_new("conference");

            if(xode_get_type(cur) != XODE_TYPE_TAG)
                continue;

            if(old == NULL)
            { /* nwe item, add it */
                int row;
                char *data[1];
                xode new;
                data[0] = xode_get_attrib(cur, "name");
                row = gtk_clist_append(GTK_CLIST(glade_xml_get_widget(cdata->xml, "conference_list")), data);
                new = xode_insert_tagnode(cdata->browse_result, cur); 
                xode_put_vattrib(new, "row", (void*)row);
            }
            else if(xode_get_attrib(cur, "type") != NULL)
            { /* remove */
                int list_index = (int)xode_get_vattrib(old, "row");

                gtk_clist_remove(GTK_CLIST(glade_xml_get_widget(cdata->xml, "conference_list")), list_index);
                xode_hide(old);
            }
            else
            { /* nick change */
                /* remove old */
                xode new;
                char *data[1];
                int row;
                int list_index = (int)xode_get_vattrib(old, "row");
                gtk_clist_remove(GTK_CLIST(glade_xml_get_widget(cdata->xml, "conference_list")), list_index);
                xode_hide(old);

                /* add new */
                data[0] = xode_get_attrib(cur, "name");
                row = gtk_clist_append(GTK_CLIST(glade_xml_get_widget(cdata->xml, "conference_list")), data);
                new = xode_insert_tagnode(cdata->browse_result, cur);
                xode_put_vattrib(new, "row", (void*)row);  
            }
        }

        xode_free(x);
    }

    return WR_OK;
}

wresult _join_dialog_join_clicked(const char *id, wevent evt, xode x, void *arg)
{
    conf_data cdata = (conf_data)arg;
    static long idn = 0;
    char idstr[20];
    conf_data_list cur;

    if(strcmp(xode_get_attrib(x, "type"), "error") == 0 && *xode_get_tagdata(x, "error") != 'N')
    {
        xode_pool_free(cdata->p);
        wijit_status("conference", "failed to join conference: %s", xode_get_tagdata(x, "error"));
        return WR_OK;
    }

    /* add this to the list of conferences */
    cur = xode_pool_malloco(cdata->p, sizeof(_conf_data_list));
    cur->cdata = cdata;
    cdata->list = cur;

    cur->next = cdata__list;
    if(cdata__list != NULL) cdata__list->prev = cur;
    cdata__list = cur;

    idn++;
    snprintf((char*)&idstr, 20, "conference2-%d", idn);
    cdata->wid = wijit_register(cdata->jid, WE_FROM, __join_dialog_join_clicked, (void*)cdata);
    wijit_register((char*)&idstr, WE_TAG_ID | WE_ONCE, __join_dialog_join_clicked, (void*)cdata);
    wijit_status("debug", "Joining with %s@%s/%s", cdata->channel, cdata->server, cdata->nick);
    wijit_write("<presence to='%s@%s'/>", cdata->channel, cdata->server);
    wijit_write("<iq type='set' id='%s' to='%s@%s'><query xmlns='jabber:iq:conference'><nick>%s</nick></query></iq>", (char*)&idstr, cdata->channel, cdata->server, cdata->nick);

    xode_free(x);
    return WR_OK;
}

void join_dialog_join_clicked(GtkWidget *w, gpointer data)
{
    xode_pool p;
    GladeXML *xml = (GladeXML*)data;
    conf_data cdata;
    static long id = 0;
    char idstr[20];

    gtk_widget_hide(glade_xml_get_widget(xml, "conference_join_dialog"));

    p = xode_pool_new();
    cdata = xode_pool_malloco(p, sizeof(_conf_data));
    cdata->p = p;
    cdata->server = xode_pool_strdup(p, gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "conference_join_server"))));
    cdata->channel = xode_pool_strdup(p, gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "conference_join_channel"))));
    cdata->nick = xode_pool_strdup(p, gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget(xml, "conference_join_nick")), 0, gtk_text_get_length(GTK_TEXT(glade_xml_get_widget(xml, "conference_join_nick")))));
    cdata->jid = xode_spool_str(p, cdata->channel, "@", cdata->server, p);

    if(*cdata->server == '\0' || *cdata->channel == '\0' || *cdata->nick == '\0')
    {
        xode_pool_free(p);
        wijit_status("conference", "insuficient data to join conference");
        return;
    }

    /* attempt negotiation with the conference server */
    id++;
    snprintf((char*)&idstr, 20, "conference%d", id);
    wijit_register((char*)&idstr, WE_TAG_ID | WE_ONCE, _join_dialog_join_clicked, (void*)cdata);
    wijit_write("<iq type='get' to='%s@%s' id='%s'><query xmlns='jabber:iq:conference'/></iq>", cdata->channel, cdata->server, (char*)&idstr);
}

void conf_menu_activate(GtkWidget *w, gpointer data)
{
    if(strcmp((char*)data, "join") == 0)
    {
        /* Display the Join Dialog */
        if(join__dialog == NULL)
        {
            xode_pool p = xode_pool_new();
            join__dialog = glade_xml_new(xode_spool_str(p, PLUGIN_DIR, "/conference.glade", p), "conference_join_dialog");
            xode_pool_free(p);

            glade_xml_signal_connect_data(join__dialog, "on_conference_join_ok_clicked", join_dialog_join_clicked, (gpointer)join__dialog);
            glade_xml_signal_connect_data(join__dialog, "on_conference_join_cancle_clicked", gtk_widget_hide, (gpointer)glade_xml_get_widget(join__dialog, "conference_join_dialog"));
        }
        
        gtk_widget_show(glade_xml_get_widget(join__dialog, "conference_join_dialog"));
    }
}

wresult _conf_connect(const char *id, wevent evt, xode data, void *arg)
{
    if(evt & WE_DISCONNECT)
    {
        conf_data_list cur = cdata__list;
        for(; cur != NULL; cur = cur->next)
        {
            _part_chat_room(NULL, cur->cdata);
        }
        gtk_widget_set_sensitive(glade_xml_get_widget(conference__xml, "conference_join"), FALSE);

        return WR_OK;
    }
    if(evt & WE_CONNECT)
    {
        gtk_widget_set_sensitive(glade_xml_get_widget(conference__xml, "conference_join"), TRUE);
        return WR_OK;
    }
    return WR_OK;
}

void init(GladeXML *xml)
{
    /* create the Conference Menu */
    xode_pool p = xode_pool_new();
    GtkWidget *menubar = glade_xml_get_widget(xml, "wijit_main_menubar");
    GtkWidget *conf_item;

    conference__xml = glade_xml_new(xode_spool_str(p, PLUGIN_DIR, "/conference.glade", p), "conference_menu");
    xode_pool_free(p);
    conf_item = glade_xml_get_widget(conference__xml, "conference_menu");

    glade_xml_signal_connect_data(conference__xml, "on_conference_join_activate", conf_menu_activate, (gpointer)"join");

    gtk_widget_show(conf_item);
    gtk_menu_bar_append(GTK_MENU_BAR(menubar), conf_item);

    wijit_register(NULL, WE_CONNECT | WE_DISCONNECT, _conf_connect, NULL);

    wijit_add_to_tree("Conference Plugin");
    wijit_status("conference","Plugin Loaded");
}
