#include <wijit.h>

extern GladeXML *WIJIT__XML;

void on_wijit_chat_entry_activate(GtkWidget *w, gpointer data) { /* do nothing */ }
void on_wijit_chat_close_clicked(GtkWidget *w, gpointer data) { /* do nothing */ }

void _send_msg(GtkWidget *w, gpointer data)
{
    GladeXML *xml = (GladeXML *)data;
    GtkWidget *entry = glade_xml_get_widget(xml, "wijit_chat_entry");
    GtkWidget *text = glade_xml_get_widget(xml, "wijit_chat_text");
    char *to;
    char *msg = gtk_entry_get_text(GTK_ENTRY(entry));

    gtk_label_get(GTK_LABEL(glade_xml_get_widget(xml, "wijit_chat_hidden")), &to);

    /* send the message */
    wijit_write("<message to='%s' type='chat'><body>%s</body></message>", to, msg);

    gtk_text_set_point(GTK_TEXT(text), gtk_text_get_length(GTK_TEXT(text)));
    gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, "<", 1);
    gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(WIJIT__XML, "wijit_connect_username"))), strlen(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(WIJIT__XML, "wijit_connect_username")))));
    gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, "> ", 2);
    if(msg != NULL)
        gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, msg, strlen(msg));
    gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, "\n", 1);

    gtk_entry_set_text(GTK_ENTRY(entry), "");
}

wresult _chat_func(const char *id, wevent evt, xode x, void *arg)
{
    GladeXML *xml = (GladeXML *)arg;

    if(*xode_get_name(x) != 'm' && *xode_get_name(x) != 'p')
    {
        xode_free(x);
        return WR_OK;
    }

    if(*xode_get_name(x) == 'm') /* message */
    {
        GtkWidget *text = glade_xml_get_widget(xml, "wijit_chat_text");
        gtk_text_set_point(GTK_TEXT(text), gtk_text_get_length(GTK_TEXT(text)));
        gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, "<", 1);
        gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, xode_get_attrib(x, "from"), strlen(xode_get_attrib(x, "from")));
        gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, "> ", 2);
        gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, xode_get_tagdata(x, "body"), xode_get_tagdata(x, "body") ? strlen(xode_get_tagdata(x, "body")) : 0);
    }

    /* presence */
    xode_free(x);
    return WR_OK;
}

GladeXML *wijit_chat_new_window()
{
    GladeXML *xml = glade_xml_new("/usr/local/lib/wijits/wijit.glade", "wijit_chat_window");
    glade_xml_signal_connect_data(xml, "on_wijit_chat_entry_activate", _send_msg, (void*)xml);
    return xml;
}

GladeXML *wijit_chat_new_interface()
{
    GladeXML *xml = glade_xml_new("/usr/local/lib/wijits/wijit.glade", "wijit_chat_interface");
    glade_xml_signal_connect_data(xml, "on_wijit_chat_entry_activate", _send_msg, (void*)xml);
    return xml;
}

int wijit_chat_attach_jid(GladeXML *xml, const char *jid)
{
    xode_pool p = xode_pool_new();
    GtkWidget *hidden = glade_xml_get_widget(xml, "wijit_chat_hidden");
    gtk_label_set_text(GTK_LABEL(hidden), jid);
    hidden = glade_xml_get_widget(xml, "wijit_chat_label");
    gtk_label_set_text(GTK_LABEL(hidden), xode_spool_str(p, "Chatting With ", jid, p));
    hidden = glade_xml_get_widget(xml, "wijit_chat_window");
    xode_pool_free(p);
    if(hidden != NULL)
        gtk_window_set_title(GTK_WINDOW(hidden), jid);
    return wijit_register(jid, WE_FROM, _chat_func, (void*)xml);
}

void wijit_chat_detach_jid(GladeXML *xml, int id)
{
    GtkWidget *hidden = glade_xml_get_widget(xml, "wijit_chat_hidden");
    gtk_label_set_text(GTK_LABEL(hidden), NULL);
    hidden = glade_xml_get_widget(xml, "wijit_chat_label");
    gtk_label_set_text(GTK_LABEL(hidden), "Chatting With (NULL)");
    hidden = glade_xml_get_widget(xml, "wijit_chat_window");
    if(hidden != NULL)
        gtk_window_set_title(GTK_WINDOW(hidden), "(Unattached Chat)");
    wijit_unregister(id);
}

typedef struct wijit_chat_struct
{
    xode_pool p;
    GladeXML *xml;
    int wid;
} _wijit_chat_struct, *wijit_chat_struct;

void _wijit_chat_new_destroy(GtkWidget *w, gpointer data)
{
    wijit_chat_struct wcs = (wijit_chat_struct)data;
    wijit_chat_detach_jid(wcs->xml, wcs->wid);
    gtk_widget_destroy(glade_xml_get_widget(wcs->xml, "wijit_chat_window"));
    xode_pool_free(wcs->p);
}

/* the happy fun ball */
GladeXML *wijit_chat_new(const char *jid)
{
    xode_pool p = xode_pool_new();
    wijit_chat_struct wcs = xode_pool_malloco(p, sizeof(_wijit_chat_struct));
    wcs->xml = wijit_chat_new_window();
    wcs->wid = wijit_chat_attach_jid(wcs->xml, jid);
    glade_xml_signal_connect_data(wcs->xml, "on_wijit_chat_close_clicked", _wijit_chat_new_destroy, (gpointer)wcs);
    gtk_widget_show(glade_xml_get_widget(wcs->xml, "wijit_chat_window"));
    return wcs->xml;
}
