#include <wijit.h>

extern GladeXML *WIJIT__XML;
extern int WIJIT_FD;

typedef struct plugin_list_st
{
    xode_pool p;
    char *filename;
    struct plugin_list_st *next;
} _plugin_list, *plugin_list;
plugin_list plugin__list = NULL;

/* 
   dispatch.c 
*/

#define ID_MATCH(x, id) (id != NULL && xode_get_attrib(x, "id") != NULL && strcmp(id, xode_get_attrib(x, "id")) == 0)
#define QUERY_MATCH(x, id) (id != NULL && xode_get_attrib(xode_get_tag(x, "query"), "id") != NULL && strcmp(id, xode_get_attrib(xode_get_tag(x, "query"), "id")) == 0)
#define X_MATCH(x, id) (id != NULL && xode_get_attrib(xode_get_tag(x, "x"), "id") != NULL && strcmp(id, xode_get_attrib(xode_get_tag(x, "x"), "id")) == 0)
#define PRESENCE_MATCH(x) (xode_get_name(x) != NULL && strcmp("presence", xode_get_name(x)) == 0)
#define MESSAGE_MATCH(x) (xode_get_name(x) != NULL && strcmp("presence", xode_get_name(x)) == 0)
#define FROM_MATCH(x, id) (id != NULL && xode_get_attrib(x, "from") != NULL && strncmp(id, xode_get_attrib(x, "from"), strlen(id)) == 0)



typedef struct wijit_dispatch_st
{
    char *id;
    wresult wid;
    wevent  evt;
    wcallback f;
    void *arg;
    struct wijit_dispatch_st *next;
    xode_pool p;
} _wdsp, *wdsp;

wdsp wijit__dispatch = NULL;

/* register a consumer or suplier of namespaces.. */
wresult wijit_register(const char *id, wevent evt, wcallback f, void *f_arg)
{
    wdsp cur;
    xode_pool p;
    static long wid = 5;

    if(f == NULL || evt == 0)
        return WR_NOK;

    /* just attach it.. who cares if they get doubles..
     * its theri fault. */
    p = xode_pool_new();
    cur = xode_pool_malloco(p, sizeof(_wdsp));
    
    cur->evt = evt;
    cur->id = xode_pool_strdup(p,id);
    cur->p = p;
    cur->f = f;
    cur->arg = f_arg;
    cur->wid = wid++;

    cur->next = wijit__dispatch;
    wijit__dispatch = cur;

    /* Hack... if this is a register for CONNECT, or DISCONNECT
     * service this call right away if it's true */
    if(evt & WE_CONNECT && WIJIT_FD != 0)
        (*cur->f)(NULL, WE_CONNECT, NULL, cur->arg);
    else if(evt & WE_DISCONNECT && WIJIT_FD == 0)
        (*cur->f)(NULL, WE_DISCONNECT, NULL, cur->arg);

    return cur->wid;
}

/* removes someone from being a listener of data */
wresult wijit_unregister(wresult id)
{
    wdsp last, cur = wijit__dispatch;

    for(last = NULL; cur != NULL; last = cur, cur = cur->next)
    {
        if(cur->wid == id)
        {
            if(last == NULL)
                wijit__dispatch = wijit__dispatch->next;
            else
                last->next = cur->next;

            xode_pool_free(cur->p);
            return WR_OK;
        }
    }

    return WR_NOK;
}

/* writes an xmlnode to a consumer */
wresult wijit_supply(wevent evt, xode data)
{
    wdsp cur = wijit__dispatch;
    int flag = 0;

    if(data == NULL)
        for(; cur != NULL; cur = cur->next)
        { /* find a matching event */
            if(evt & cur->evt)
            {
                (*cur->f)(NULL, evt, NULL, cur->arg);
                flag = 1;
            }
        }
    else
        for(; cur != NULL; cur = cur->next)
        {
            if(cur->evt & WE_TAG_ID)
            {
                if(ID_MATCH(data, cur->id))
                {
                    (*cur->f)(cur->id, WE_TAG_ID, xode_dup(data), cur->arg);
                    if(cur->evt & WE_ONCE)
                    {
                        wijit_unregister(cur->wid);
                    }
                    flag = 1;
                }
            }
            if(cur->evt & WE_X_XMLNS)
            {
                if(X_MATCH(data, cur->id))
                {
                    (*cur->f)(cur->id, WE_X_XMLNS, xode_dup(data), cur->arg);
                    if(cur->evt & WE_ONCE)
                    {
                        wijit_unregister(cur->wid);
                    }
                    flag = 1;
                }
            }
            if(cur->evt & WE_QUERY_XMLNS)
            {
                if(QUERY_MATCH(data, cur->id));
                {
                    (*cur->f)(cur->id, WE_QUERY_XMLNS, xode_dup(data), cur->arg);
                    if(cur->evt & WE_ONCE)
                    {
                        wijit_unregister(cur->wid);
                    }
                    flag = 1;
                }
            }
            if(cur->evt & WE_PRESENCE)
            {
                if(PRESENCE_MATCH(data))
                {
                    (*cur->f)(NULL, WE_PRESENCE, xode_dup(data), cur->arg);
                    if(cur->evt & WE_ONCE)
                    {
                        wijit_unregister(cur->wid);
                    }
                    flag = 1;
                }
            }
            if(cur->evt & WE_MESSAGE)
            {
                if(MESSAGE_MATCH(data))
                {
                    (*cur->f)(NULL, WE_MESSAGE, xode_dup(data), cur->arg);
                    if(cur->evt & WE_ONCE)
                    {
                        wijit_unregister(cur->wid);
                    }
                    flag = 1;
                }
            }
            if(cur->evt & WE_FROM)
            {
                if(FROM_MATCH(data, cur->id))
                {
                    (*cur->f)(cur->id, WE_FROM, xode_dup(data), cur->arg);
                    if(cur->evt & WE_ONCE)
                    {
                        wijit_unregister(cur->wid);
                    }
                    flag = 1;
                }
            }
        }

    if(flag)
        return WR_OK;

    return WR_NOK;
}

void *_wijit_loader(const char *file)
{
    void *so_h;
    char *dlerr;

    if(file == NULL)
        return NULL;

    so_h = dlopen(file, RTLD_LAZY);
    dlerr = dlerror();
    if(dlerr)
    {
        wijit_status("LOAD", "Failed to load %s: %s", file, dlerr);
        return NULL;
    }

    return so_h;
}

void *_wijit_symbol(const char *file)
{
    void *so_h = _wijit_loader(file);
    void (*func_h)(GladeXML *xml);
    char *dlerr;

    if(so_h == NULL)
        return NULL;

    func_h = dlsym(so_h, "init");
    dlerr = dlerror();
    if(dlerr)
    {
        wijit_status("LOAD", "Failed to load Symbol 'init' in file %s", file);
        return NULL;
    }

    return func_h;
}

wresult wijit_load_plugin(const char *file)
{
    void (*func_h)(GladeXML *xml) = _wijit_symbol(file);
    plugin_list cur = plugin__list;
    xode_pool p;

    if(func_h == NULL)
        return WR_NOK;

    for(; cur != NULL; cur = cur->next)
    {
        if(strcmp(file, cur->filename) == 0)
            break;
    }

    if(cur != NULL)
    { /* already loaded */
        wijit_status("load","This Plugin is already loaded");
        return WR_NOK;
    }

    p = xode_pool_new();
    cur = xode_pool_malloco(p, sizeof(_plugin_list));
    cur->p = p;
    cur->filename = xode_pool_strdup(p, file);
    cur->next = plugin__list;
    plugin__list = cur;

    (*func_h)(WIJIT__XML);
    return WR_OK;
}
