#include <wijit.h>

xode WIJIT_PREFS = NULL;
extern GladeXML *WIJIT__XML;

xode wijit_prefs_get(void)
{
    xode_pool p;

    /* return the cached copy, if already loaded */
    if(WIJIT_PREFS != NULL)
        return WIJIT_PREFS;

    /* make sure directory exists */
    p = xode_pool_new();
    mkdir(xode_spool_str(p, getenv("HOME"), "/.wijit/", p), 00600);
    chmod(xode_spool_str(p, getenv("HOME"), "/.wijit/", p), S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP);
    xode_pool_free(p);

    /* read in xode file */
    WIJIT_PREFS = xode_from_file("~/.wijit/prefs.xml");

    /* if NULL, create an empty container */
    if(WIJIT_PREFS == NULL)
    {
        xode login;
        WIJIT_PREFS = xode_new("wijit_prefs");        
        login = xode_insert_tag(WIJIT_PREFS, "login");
        xode_insert_cdata(xode_insert_tag(login, "username"), gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(WIJIT__XML, "wijit_connect_username"))), -1);
        xode_insert_cdata(xode_insert_tag(login, "password"), gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(WIJIT__XML, "wijit_connect_password"))), -1);
        xode_insert_cdata(xode_insert_tag(login, "resource"), gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(WIJIT__XML, "wijit_connect_resource"))), -1);
        xode_insert_cdata(xode_insert_tag(login, "server"), gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(WIJIT__XML, "wijit_connect_server"))), -1);
        xode_put_attrib(xode_insert_tag(WIJIT_PREFS, "plugins"), "dir", PLUGIN_DIR);
    }

    return WIJIT_PREFS;
}

void wijit_prefs_save(void)
{
    if(WIJIT_PREFS == NULL)
        WIJIT_PREFS = wijit_prefs_get();

    /* write out the xode file */
    xode_to_file("~/.wijit/prefs.xml", WIJIT_PREFS);
}

void on_wijit_prefs_okay_clicked(GtkWidget *w, gpointer data)
{
    /* update preferences for current profile */
    xode x = wijit_prefs_get(), cur_x;
    char *plugins, *cur, *last;
    xode_hide(xode_get_tag(x, "login/username"));
    xode_hide(xode_get_tag(x, "login/resource"));
    xode_hide(xode_get_tag(x, "login/server"));
    xode_hide(xode_get_tag(x, "login/password"));

    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(WIJIT__XML, "wijit_prefs_login"))))
    {
        /* rewrite the pref */
        xode_insert_cdata(xode_insert_tag(xode_get_tag(x, "login"), "username"), gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(WIJIT__XML, "wijit_connect_username"))),-1);
        xode_insert_cdata(xode_insert_tag(xode_get_tag(x, "login"), "resource"), gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(WIJIT__XML, "wijit_connect_resource"))),-1);
        xode_insert_cdata(xode_insert_tag(xode_get_tag(x, "login"), "server"), gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(WIJIT__XML, "wijit_connect_server"))),-1);
    }

    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(WIJIT__XML, "wijit_prefs_password"))))
    {
        /* rewrite the pref */
        xode_insert_cdata(xode_insert_tag(xode_get_tag(x, "login"), "password"), gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(WIJIT__XML, "wijit_connect_password"))), -1);
    }

    /* save plugin data */
    /* clear all plugins from file */
    while((cur_x = xode_get_tag(x, "plugins/plugin")) != NULL) xode_hide(cur_x);

    w = glade_xml_get_widget(WIJIT__XML, "wijit_prefs_plugins");
    last = plugins = (char*)strdup((char*)gtk_editable_get_chars(GTK_EDITABLE(w),0,gtk_text_get_length(GTK_TEXT(w))));
    if(last != NULL && *last != '\0')
    {
        while((cur = strchr(last, '\n')) != NULL)
        {
            cur[0] = '\0';
            cur++;
            xode_insert_cdata(xode_insert_tag(xode_get_tag(x, "plugins"), "plugin"), last, -1);
            last = cur;
        }
        if(*last != '\0')
        xode_insert_cdata(xode_insert_tag(xode_get_tag(x, "plugins"), "plugin"), last, -1);
    }
    if(plugins != NULL)
        free(plugins);

    /* write to file */
    wijit_prefs_save();

    gtk_widget_hide(glade_xml_get_widget(WIJIT__XML, "wijit_prefs_dialog"));
}

void on_wijit_main_menu_wijit_preferences_activate(GtkWidget *warg, gpointer arg)
{
    /* read configuration file */
     xode x = wijit_prefs_get(), cur;
     char *data;
     GtkWidget *w;

    /* set preference window items */
    data = xode_get_attrib(xode_get_tag(x, "plugins"), "dir");
    w = glade_xml_get_widget(WIJIT__XML, "wijit_prefs_dir");
    gtk_entry_set_text(GTK_ENTRY(w), data);

    w = glade_xml_get_widget(WIJIT__XML, "wijit_prefs_plugins");
    gtk_text_set_point(GTK_TEXT(w), gtk_text_get_length(GTK_TEXT(w)));
    gtk_text_backward_delete(GTK_TEXT(w), gtk_text_get_length(GTK_TEXT(w)));
    for(cur = xode_get_firstchild(xode_get_tag(x, "plugins")); cur != NULL; cur = xode_get_nextsibling(cur))
    {
        char *plugin;
        if(xode_get_type(cur) != XODE_TYPE_TAG) continue;
        if(*xode_get_name(cur) != 'p') continue;

        plugin = xode_get_data(cur);
        if(plugin != NULL)
        {
            gtk_text_insert(GTK_TEXT(w), NULL, NULL, NULL, plugin, strlen(plugin));
            gtk_text_insert(GTK_TEXT(w), NULL, NULL, NULL, "\n", 1);
        }
    }
    
    data = xode_get_tagdata(x, "login/username");
    if(data != NULL && *data != '\0')
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(WIJIT__XML, "wijit_prefs_login")), TRUE);

    data = xode_get_tagdata(x, "login/password");
    if(data != NULL && *data != '\0')
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(WIJIT__XML, "wijit_prefs_password")), TRUE);

    /* load plugin tabs */

    /* display window */
    gtk_widget_show(glade_xml_get_widget(WIJIT__XML, "wijit_prefs_dialog"));
}
