#include <wijit.h>

GladeXML *WIJIT__XML = NULL;
extern gint WIJIT_FD_TAG;
extern int  WIJIT_FD;
extern xode_pool WIJIT_XS_POOL;

void wijit_add_to_tree(const char *name)
{
    GtkWidget *tree_item = gtk_tree_item_new_with_label(name);
    gtk_widget_show(tree_item);

    gtk_tree_append(GTK_TREE(glade_xml_get_widget(WIJIT__XML, "wijit_notebook_plugintree")), tree_item);
}

void wijit_status(const char *from, const char *msg, ...)
{
    va_list va;
    xode_pool p;
    char txt[1000];
    char *text;
    GtkWidget *status = glade_xml_get_widget(WIJIT__XML, "wijit_statusbox");  

    p = xode_pool_new();
    va_start(va, msg);
    vsnprintf((char*)&txt, 1000, msg, va);
    va_end(va);
    text = xode_spool_str(p, "[ ", from, " ] ", (char*)&txt, "\n", p);

    gtk_text_set_point(GTK_TEXT(status), gtk_text_get_length(GTK_TEXT(status)));
    gtk_text_insert(GTK_TEXT(status),NULL, NULL, NULL, text , strlen(text));
    xode_pool_free(p);
}

void on_wijit_main_menu_wijit_connect_activate(GtkWidget *w, gpointer data)
{ 
    wijit_show_connect_dialog();
}

void on_wijit_main_menu_wijit_disconnect_activate(GtkWidget *w, gpointer data)
{
    GtkWidget *Connect = glade_xml_get_widget(WIJIT__XML, "wijit_main_menu_wijit_connect");
    GtkWidget *disconnect = glade_xml_get_widget(WIJIT__XML, "wijit_main_menu_wijit_disconnect");
    gtk_widget_set_sensitive(Connect, TRUE);
    gtk_widget_set_sensitive(disconnect, FALSE);
    gdk_input_remove(WIJIT_FD_TAG);
    close(WIJIT_FD);
    xode_pool_free(WIJIT_XS_POOL);
    WIJIT_XS_POOL = NULL;
    WIJIT_FD = 0;
    wijit_status("IO", "Disconnected");
    wijit_supply(WE_DISCONNECT, NULL);
}

void on_wijit_main_menu_wijit_clear_status_window_activate(GtkWidget *w, gpointer data)
{
    GtkWidget *status = glade_xml_get_widget(WIJIT__XML, "wijit_statusbox");  
    gtk_text_set_point(GTK_TEXT(status), gtk_text_get_length(GTK_TEXT(status)));
    gtk_text_backward_delete(GTK_TEXT(status),gtk_text_get_length(GTK_TEXT(status))); 
}

void on_wijit_plugin_dialog_load_from_file_clicked(GtkWidget *w, gpointer data)
{
    GtkWidget *dialog = glade_xml_get_widget(WIJIT__XML, "wijit_file_select");
    gtk_widget_show(dialog);
}

void _on_wijit_main_menu_wijit_load_plugin(GtkWidget *w, gpointer data)
{
    GtkWidget *dialog = glade_xml_get_widget(WIJIT__XML, "wijit_file_select");

    gtk_widget_hide(dialog);
    if(wijit_load_plugin(gtk_file_selection_get_filename(GTK_FILE_SELECTION(dialog))) == WR_NOK)
    {
        wijit_status("PLUGIN", "FAILED TO LOAD PLUGIN");
        return;
    }

    wijit_status("PLUGIN", "Loaded Plugin");
}

char *Selected = NULL;

void on_wijit_plugin_dialog_list_select_row(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
    gtk_clist_get_text(GTK_CLIST(widget),row,1, &Selected);
}

void on_wijit_plugin_dialog_load_clicked(GtkWidget *w, gpointer data)
{
    if(wijit_load_plugin(Selected) == WR_NOK)
    {
        wijit_status("PLUGIN", "FAILED TO LOAD PLUGIN");
        return;
    }

    wijit_status("PLUGIN", "Loaded Plugin");
    
}

void on_wijit_main_menu_wijit_load_plugin_activate(GtkWidget *w, gpointer data)
{
    DIR *dir;
    struct dirent *entry;
    GtkWidget *dialog = glade_xml_get_widget(WIJIT__XML, "wijit_plugin_dialog");

    /* load a list of plugins */
    gtk_clist_clear(GTK_CLIST(glade_xml_get_widget(WIJIT__XML, "wijit_plugin_dialog_list")));
    dir = opendir("/usr/local/lib/wijits");
    while((entry = readdir(dir)) != NULL)
    {
        char *f_name = strdup(entry->d_name);
        char *_so = strstr(f_name, ".so");
        char *list_entry[2];
        char full_name[255];
        if(_so == NULL)
            continue;
        _so[0] = '\0';
        list_entry[0] = f_name;
        snprintf((char*)&full_name, 255, "/usr/local/lib/wijits/%s.so", list_entry[0]);
        list_entry[1] = strdup((char*)&full_name);

        gtk_clist_append(GTK_CLIST(glade_xml_get_widget(WIJIT__XML, "wijit_plugin_dialog_list")), list_entry);

    }
    gtk_widget_show(dialog);
}

int main(int argc, char **argv)
{
    xode_pool p = xode_pool_new();
    xode x, cur;
    char *data;
    GtkWidget *w;
    gtk_init(&argc, &argv);

    glade_init();

    /* load the interface */
    WIJIT__XML = glade_xml_new(xode_spool_str(p, PLUGIN_DIR, "/wijit.glade", p), NULL);

    /* connect the signals in the interface */
    glade_xml_signal_autoconnect(WIJIT__XML);
    /* start the event loop */

    wijit_add_to_tree("Wijit Default Plugin");

    /* load prefs, and set up connect dialog */

    x = wijit_prefs_get();
    data = xode_get_tagdata(x, "login/username");
    w = glade_xml_get_widget(WIJIT__XML, "wijit_connect_username");
    if(data != NULL)
        gtk_entry_set_text(GTK_ENTRY(w), data);

    data = xode_get_tagdata(x, "login/password");
    w = glade_xml_get_widget(WIJIT__XML, "wijit_connect_password");
    if(data != NULL)
        gtk_entry_set_text(GTK_ENTRY(w), data);

    data = xode_get_tagdata(x, "login/resource");
    w = glade_xml_get_widget(WIJIT__XML, "wijit_connect_resource");
    if(data != NULL)
        gtk_entry_set_text(GTK_ENTRY(w), data);

    data = xode_get_tagdata(x, "login/server");
    w = glade_xml_get_widget(WIJIT__XML, "wijit_connect_server");
    if(data != NULL)
        gtk_entry_set_text(GTK_ENTRY(w), data);

    /* load any default plugins */
    data = xode_get_attrib(xode_get_tag(x, "plugins"), "dir");
    for(cur = xode_get_firstchild(xode_get_tag(x, "plugins")); data != NULL && cur != NULL; cur = xode_get_nextsibling(cur))
    {
        if(xode_get_type(cur) != XODE_TYPE_TAG) continue;

        wijit_load_plugin(xode_spool_str(p, data, "/", xode_get_data(cur), p));
    }

    /* main handler */
    gtk_main();

    /* write prefs to file */
    wijit_prefs_save();
    xode_pool_free(p);
    return 0;
}
